#include <Peggy2.h>
#include <avr/pgmspace.h>
#include "Characters.h"

// Frame split into 3 sub-frames (for 8 brightness levels)
Peggy2 frame4;
Peggy2 frame2;
Peggy2 frame1;

Peggy2 walls;
Peggy2 apples;


// Define game area
#define gameMinX 0
#define gameMaxX 24
#define gameMinY 0
#define gameMaxY 23
#define gameCentreX 12
#define gameCentreY 12

// Define score area
#define scoreMinX 0
#define scoreMaxX 24
#define scoreY 24

#define SNAKE 0
#define PONG 1
#define TWOPONG 2
#define BREAKOUT 3
#define RACE 4
#define DEMO 5
#define GAMES 6
#define SCORE 6
#define LEVEL 7
#define GAME 8
#define OVER 9

#define NONE 0xFF
byte gameType = NONE;
prog_char string_0[] PROGMEM = "SNAKE";
prog_char string_1[] PROGMEM = "PONG";
prog_char string_2[] PROGMEM = "2PONG";
prog_char string_3[] PROGMEM = "BREAK";
prog_char string_4[] PROGMEM = "RACE";
prog_char string_5[] PROGMEM = "DEMO";
prog_char string_6[] PROGMEM = "SCORE";
prog_char string_7[] PROGMEM = "LEVEL";
prog_char string_8[] PROGMEM = "GAME";
prog_char string_9[] PROGMEM = "OVER";

PROGMEM const char *stringTable[] =
{   
  string_0,
  string_1,
  string_2,
  string_3,
  string_4,
  string_5,
  string_6,
  string_7,
  string_8,
  string_9
};


int points=0;
byte level=1;
#define levelMax 25 // Score to go to next level
byte speed=30;

#define speakerPin 19
#define ENDTUNE 0
#define RICKTUNE 1
#define SMOOTHTUNE 2

void setup()
{
  frame4.HardwareInit();   // Call this once to init the hardware. 

  // Setup buttons
  DDRB = 0xFE;    // PORTB pin 0 is button 0, set it up as an input
  PORTB |= 0x01;  // PORTB pin 0 pull up
  DDRC = 0xE0;    // PORTC pins 0 - 4 set up as inputs corresponding to buttons 1 - 5
  PORTC |= 0x1F;  // PORTC pins 0 - 4 pull ups
}

void loop()
{
  if (gameType == SNAKE) {
    snake_loop();
  } 
  else if (gameType==PONG || gameType==TWOPONG) {
    pong_loop();
  } 
  else if (gameType==BREAKOUT) {
    breakout_loop();
  } 
  else if (gameType==RACE) {
    race_loop();
  } 
  else if (gameType==DEMO) {
    demo_loop();
  } 
  else {
    display_intro();
  }
}

void setup_game()
{
  if (gameType == SNAKE) {
    snake_setup_game();
  } 
  else if (gameType == PONG || gameType==TWOPONG) {
    pong_setup_game();
  } 
  else if (gameType == BREAKOUT) {
    breakout_setup_game();
  } 
  else if (gameType == RACE) {
    race_setup_game();
  } 
  else if (gameType == DEMO) {
    demo_setup_game();
  } 
  else {
    display_intro();
  }
}


void add_wall(byte x1, byte y1,byte x2, byte y2)
{
  setLine(x1,y1,x2,y2,7);
  walls.Line(x1,y1,x2,y2);
}

void update_scorebar()
{
  byte mypoints = points % (levelMax);
  for (byte i=scoreMinX;i<=scoreMaxX;i++) {
    if ((i-scoreMinX) < mypoints) {
      setPixel(i,scoreY,7);
    } 
    else {
      setPixel(i,scoreY,0);
    }
  }  
}

void display_score()
{
  restart_melody();
  for (byte j=0;j<=24;j++) {
    playMelody(RICKTUNE);
    scroll_up(false);

    write_predef(SCORE,0,0+j-24,7);
    write_number(points,0,7+j-24,7);

    write_predef(LEVEL,1,18+j-24,7);
    write_number(level,16,11+j-24,7);
    refresh_display(10);
  }

  while((PINC & 0x1F) == 0x1F) {  // Wait for button 1-5
    refresh_display(10);
    playMelody(RICKTUNE);
  }
}

void display_gameover()
{
  restart_melody();
  for (byte j=gameMaxX;j>gameMinX;j--) {
    playMelody(ENDTUNE);
    scroll_down(false);
    write_predef(GAME,0,0+j,7);
    write_predef(OVER,4,10+j,7);
    refresh_display(2);
  }
  boolean playing;

  while(playMelody(ENDTUNE)) {
    refresh_display(10);
  }
}

void scroll_up(boolean wrap)
{
  unsigned long f4,f2,f1;
  if (wrap) {
    f4=frame4.buffer[gameMaxY];
    f2=frame2.buffer[gameMaxY];
    f1=frame1.buffer[gameMaxY];
  }

  for (byte i=gameMaxY;i>gameMinY;i--){
    frame4.buffer[i]=frame4.buffer[i-1];
    frame2.buffer[i]=frame2.buffer[i-1];
    frame1.buffer[i]=frame1.buffer[i-1];
  }
  if (wrap) {
    frame4.buffer[gameMinY]=f4;
    frame2.buffer[gameMinY]=f2;
    frame1.buffer[gameMinY]=f1;
  } 
  else {
    frame4.buffer[gameMinY]=0;
    frame2.buffer[gameMinY]=0;
    frame1.buffer[gameMinY]=0;
  }
}

void scroll_down(boolean wrap)
{
  unsigned long f4,f2,f1;
  if (wrap) {
    f4=frame4.buffer[gameMinY];
    f2=frame2.buffer[gameMinY];
    f1=frame1.buffer[gameMinY];
  }

  for (byte i=gameMinY;i<gameMaxY;i++){
    frame4.buffer[i]=frame4.buffer[i+1];
    frame2.buffer[i]=frame2.buffer[i+1];
    frame1.buffer[i]=frame1.buffer[i+1];
  }
  if (wrap) {
    frame4.buffer[gameMaxY]=f4;
    frame2.buffer[gameMaxY]=f2;
    frame1.buffer[gameMaxY]=f1;
  } 
  else {
    frame4.buffer[gameMaxY]=0;
    frame2.buffer[gameMaxY]=0;
    frame1.buffer[gameMaxY]=0;
  }
}

void display_intro()
{
  restart_melody();
  clear_display();
  byte selectDir=0;
  gameType=0;
  write_predef((gameType+GAMES-1) % GAMES,1,1,2);
  write_predef(gameType,1,9,7);
  write_predef((gameType+1) % GAMES,1,17,2);

  while ((PINC & 0x01) && (PINB & 0x1)) {  // Wait for button 1 or select button
    playMelody(SMOOTHTUNE);
    selectDir=0;
    if (!(PINC & 0x04)) selectDir=1; // UP
    if (!(PINC & 0x08)) selectDir=2; // DOWN
    if (selectDir) {
      if (selectDir==1) {
        gameType=(gameType+1) % GAMES;
      } 
      else {
        gameType=(gameType+GAMES-1) % GAMES;
      }
      for (byte k=0; k<8 ; k++) {
        playMelody(SMOOTHTUNE);
        clear_display();
        if(selectDir == 1) {
          byte i=8-k;
          write_predef((gameType+GAMES-2) % GAMES,1,-7+i,2);
          write_predef((gameType+GAMES-1) % GAMES,1,1+i,2);
          write_predef(gameType,1,9+i,2);
          write_predef((gameType+1) % GAMES,1,17+i,2);
        } 
        else {
          write_predef((gameType+GAMES-1) % GAMES,1,-7+k,2);
          write_predef(gameType,1,1+k,2);
          write_predef((gameType+1) % GAMES,1,9+k,2);
          write_predef((gameType+2) % GAMES,1,17+k,2);

        }
        refresh_display(5);
      }

      clear_display();
      write_predef((gameType+GAMES-1) % GAMES,1,1,2);
      write_predef(gameType,1,9,7);
      write_predef((gameType+1) % GAMES,1,17,2);
    }
    refresh_display(1);
  }
  randomSeed(millis()); // Gives a pretty good random value
  setup_game();
}

void setPixel(byte x, byte y, byte brightness)
{
  if (x>24 || y>24) return;
  frame4.WritePoint(x,y,(brightness & 4));
  frame2.WritePoint(x,y,(brightness & 2));
  frame1.WritePoint(x,y,(brightness & 1));
}

void setLine(byte x1, byte y1, byte x2, byte y2, byte brightness)
{
  if (brightness & 4) frame4.Line(x1,y1,x2,y2);
  if (brightness & 2) frame2.Line(x1,y1,x2,y2);
  if (brightness & 1) frame1.Line(x1,y1,x2,y2);
}

void refresh_display (byte repeat)
{
  for (byte i=0; i<repeat; i++) {
    frame4.RefreshAll(4);
    frame2.RefreshAll(2);
    frame1.RefreshAll(1);
  }
}

void clear_display()
{
  frame1.Clear();
  frame2.Clear();
  frame4.Clear();
}

void write_number(int num, int x, int y, byte brightness)
{
  char numChar[10];
  itoa(num, numChar, 10);
  write_string(numChar,x,y,brightness);
}

void write_string(char string[], int x, int y, byte brightness)
{
  byte i=0;
  while (string[i] != '\0') {
    show_char(string[i], x+(5*i), y, brightness);
    i++;
  }
}

void write_predef(byte stringNumber, int x, int y, byte brightness)
{
  char buffer[8];
  strcpy_P(buffer, (char*)pgm_read_word(&(stringTable[stringNumber])));
  write_string(buffer,x,y,brightness);
}

void show_char(char Character, int x, int y, byte brightness)
{
  byte charNum = 36;

  if ((Character >= '0') && (Character <= '9')) {
    charNum=Character-48;
  }
  if ((Character >= 'A') && (Character <= 'Z')) {
    charNum=Character-55;
  }

  unsigned long w;
  w = pgm_read_dword_near(chars + charNum);
  for (byte i=0; i<6; i++) {
    byte offset = 4*(5-i);
    byte b = w >> offset; // get four bits
    b=b & 0xF;
    if (b & B1000) setPixel(x,y+i,brightness);
    if (b & B0100) setPixel(x+1,y+i,brightness);
    if (b & B0010) setPixel(x+2,y+i,brightness);
    if (b & B0001) setPixel(x+3,y+i,brightness);
  }
}

