#include "pitches.h"
// Define special pong properties
int topPaddle;
int topPaddleOld;
byte topItemWidth;
byte aiPoints=0;

void pong_draw_paddles()
{
  if (topPaddleOld != topPaddle) {
    for (byte i=gameMinX;i<=gameMaxX;i++) {
      if ((i >= topPaddle) && (i<(topPaddle+topItemWidth))) {
        setPixel(i,gameMinY,7);
      } else {
        setPixel(i,gameMinY,0);
      }
    }
    topPaddleOld=topPaddle;
  }
  if (bottomItemOldX != bottomItemX) {
    for (byte i=gameMinX;i<=gameMaxX;i++) {
      if ((i >= bottomItemX) && (i<(bottomItemX+bottomItemWidth))) {
        setPixel(i,gameMaxY,7);
      } else {
        setPixel(i,gameMaxY,0);
      }
    }
    bottomItemOldX=bottomItemX;
  }
}

void pong_setup_board()
{
  clear_display();
  for (byte i=gameMinX; i< gameMaxX; i++) {
    setPixel(i,gameCentreY,3);
  }
  add_wall(gameMinX,gameMinY,gameMinX,gameMaxY);
  add_wall(gameMaxX,gameMinY,gameMaxX,gameMaxY);
}

void pong_loop()
{
  if (!(PINC & 0x02)) bottomItemX--;  // PORT C pin 1 is attached to left button
  if (!(PINC & 0x10)) bottomItemX++;  // PORT C pin 4 is attached to right button
  bottomItemX = constrain(bottomItemX,(gameMinX-bottomItemWidth+1),gameMaxX);
  
  if (gameType==TWOPONG) {
    if (!(PINC & 0x04)) topPaddle--;
    if (!(PINC & 0x08)) topPaddle++;
    topPaddle = constrain(topPaddle,(gameMinX-bottomItemWidth+1),gameMaxX);
  } else {
    pong_move_ai();
  }
  
  if (loopCount) {
    pong_move_ball();
  }
  loopCount=!loopCount;
  pong_draw_paddles();
  refresh_display(speed);
}

void pong_move_ai()
{
  // Only move 2/3 of the time
  if (random(3) != 0) {
    if (ballX < (topPaddle+(topItemWidth/2))) {
      topPaddle--;
    } 
    else {
      topPaddle++;
    }
  }
}

void pong_setup_game()
{ 
  pong_setup_board();
  topPaddle=gameCentreX;
  bottomItemX=gameCentreX;
  topPaddleOld=0;
  bottomItemOldX=0;
  topItemWidth=5;
  bottomItemWidth=5;
  ballX=gameCentreX;
  ballY=gameCentreY;
  itemDirX=random(2);
  if (itemDirX==0) itemDirX=-1;
  itemDirY=random(2);
  if (itemDirY==0) itemDirY=-1;
  pong_draw_paddles();
  if (gameType==TWOPONG) {
    speed=10-(points+aiPoints);
  } else {
    speed=10-points;
  }
  speed=min(speed,9);
}


void pong_move_ball()
{
  byte oldBallX = ballX;
  byte oldBallY = ballY;
  ballX=ballX+itemDirX;
  ballY=ballY+itemDirY;
  if ((ballX <= (gameMinX)) || (ballX >= (gameMaxX))) {
    itemDirX=-itemDirX;
    ballX=ballX+itemDirX;
    tone(speakerPin,NOTE_B4,200);
  }

  if (ballY <= (gameMinY+1)) {
    itemDirY=-itemDirY;
    if ((ballX < topPaddle) || (ballX>=(topPaddle+topItemWidth))) {
      // Missed it
      points++;
      setPixel(ballX,ballY,7);
      pong_display_score_update();
      pong_setup_game();
      update_scorebar();
    } else {
      tone(speakerPin, NOTE_A4,200);
    }
  } else if (ballY >= (gameMaxY-1)) {
    itemDirY=-itemDirY;
    if ((ballX < bottomItemX) || (ballX>=(bottomItemX+bottomItemWidth))) {
      // Missed it!
      setPixel(ballX,ballY,7);
      aiPoints++;
      pong_display_score_update();
      pong_setup_game();
      update_scorebar();
    } else {
      if (ballX<(bottomItemX+(bottomItemWidth/2)-1)) {
        itemDirX--;
      } else if (ballX>(bottomItemX+(bottomItemWidth/2)+1)) {
        itemDirX++;
      }
      tone(speakerPin, NOTE_C4,200);
    }
  }
  setPixel(ballX,ballY,7);
  if (oldBallY == gameCentreY) {
    setPixel(oldBallX,oldBallY,3);
  }  else if ((ballX == gameMinX) || (ballX == gameMaxX)) {
    setPixel(oldBallX,oldBallY,7);
  } else {
    setPixel(oldBallX,oldBallY,0);
  }
 
}

void pong_display_score_update()
{
  tone(speakerPin,NOTE_C6,200);
  write_number(aiPoints,5,3,7);
  write_number(points,5,15,7);
  
  refresh_display(200);
}
