// Define snake properties;
#define snakeMaxLength 52
int snake[snakeMaxLength];
byte snakeHead;
byte snakeLength;

void snake_loop()
{

  int newitemDirX=0;
  int newitemDirY=0;
  if (!(PINC & 0x02)) newitemDirX=-1;  // PORT C pin 1 is attached to left button
  if (!(PINC & 0x04)) newitemDirY=+1;  // PORT C pin 2 is attached to down button
  if (!(PINC & 0x08)) newitemDirY=-1;  // PORT C pin 3 is attached to up button
  if (!(PINC & 0x10)) newitemDirX=+1;  // PORT C pin 4 is attached to right button

  if (newitemDirX || newitemDirY) {
    if (!((itemDirX == -newitemDirX) && (itemDirY == -newitemDirY))) {
      itemDirX=newitemDirX;
      itemDirY=newitemDirY;
    }
  }

  snake_move();
  tuneStep=(tuneStep+1) % 2;
  tone(speakerPin,500+(points % levelMax)*50+(100*tuneStep),speed*2);
  refresh_display(speed);
}

void snake_add_apple()
{
  byte appleX = random(gameMinX+1,gameMaxX);
  byte appleY = random(gameMinY+1,gameMaxY);
  if (frame2.GetPoint(appleX,appleY) || frame4.GetPoint(appleX,appleY)) {
    snake_add_apple();
    return;
  }

  apples.SetPoint(appleX,appleY);
  setPixel(appleX,appleY,7);
}

void snake_setup_snake(byte x, byte y)
{
  for (byte i=0;i<snakeMaxLength;i++) {
    snake[i]=0xFFFF; // marker for unused spot
  }
  snakeLength=1;
  snakeHead=snakeLength-1;
  snake[snakeHead] = (x<<8) + y;
  setPixel(highByte(snake[snakeHead]),lowByte(snake[snakeHead]),7);
  itemDirX=random(3)-1;
  itemDirY=random(3)-1;
  if ((itemDirX==0) && (itemDirY==0)) itemDirX=1;
}

void snake_move ()
{
  byte oldsnakeHead = snakeHead;
  byte oldsnakeTail=(snakeMaxLength+snakeHead-snakeLength) % snakeMaxLength;
  snakeHead=(snakeHead+1) % snakeMaxLength;

  byte snakeTail=(oldsnakeTail +1) % snakeMaxLength;

  byte x = (highByte(snake[oldsnakeHead]) + itemDirX) % (gameMaxX+1);
  if ((x < gameMinX) || (x > 50)) x = gameMaxX;

  byte y = (lowByte(snake[oldsnakeHead]) + itemDirY) % (gameMaxY+1);
  if ((y < gameMinY) || (y > 50)) y = gameMaxY;

  snake[snakeHead] = (x<<8) + y;
  // Check if hitting an apple
  if (apples.GetPoint(highByte(snake[snakeHead]),lowByte(snake[snakeHead]))) {
    points++;

    if ((points % levelMax) == 0) {
      //next level
      level++;
      speed = 32-((points % levelMax)/2)-(2*level);
      display_score();
      snake_setup_level();
      return;
    }
    speed = 32-((points % levelMax)/2)-(2*level);
    snakeLength=min(snakeLength+2,snakeMaxLength);
    apples.ClearPoint(highByte(snake[snakeHead]),lowByte(snake[snakeHead]));
    snake_add_apple();
    update_scorebar();
  }

  boolean hit = false;
  // Check if hitting self
  for (byte i=0; i < snakeMaxLength; i++) {
    if (i != snakeHead) {
      if (snake[i] == snake[snakeHead]) {
        hit=true;
      }
    }
  }
  // Check if hitting wall
  if (walls.GetPoint(highByte(snake[snakeHead]),lowByte(snake[snakeHead]))) {
    hit=true;
  }

  if (hit) {
    display_gameover();
    display_score();
    snake_setup_game();
  } 
  else {
    // Set the head/tail values
    setPixel(highByte(snake[oldsnakeTail]),lowByte(snake[oldsnakeTail]),0);
    snake[oldsnakeTail] = 0xFFFF;
    setPixel(highByte(snake[snakeTail]),lowByte(snake[snakeTail]),3);
    setPixel(highByte(snake[snakeHead]),lowByte(snake[snakeHead]),7);
  }
}

void snake_setup_board()
{

  byte numLevels = 4;
  byte myLevel=((level-1) % numLevels) + 1;
  walls.Clear();
  if (myLevel == 1) {
    // Four outside walls
    add_wall(gameMinX,gameMinY,gameMaxX,gameMinY);
    add_wall(gameMaxX,gameMinY,gameMaxX,gameMaxY);
    add_wall(gameMinX,gameMinY,gameMinX,gameMaxY);
    add_wall(gameMinX,gameMaxY,gameMaxX,gameMaxY);
    snake_setup_snake(gameCentreX, gameCentreY);
  } 
  else if (myLevel == 2) {
    // Four outside walls plus internal bar
    add_wall(gameMinX,gameMinY,gameMaxX,gameMinY);
    add_wall(gameMaxX,gameMinY,gameMaxX,gameMaxY);
    add_wall(gameMinX,gameMinY,gameMinX,gameMaxY);
    add_wall(gameMinX,gameMaxY,gameMaxX,gameMaxY);
    add_wall((gameMaxX/3), gameCentreY, (gameMaxX/3*2), gameCentreY);
    snake_setup_snake(gameCentreX, (gameMaxY-gameMinY)/3);
  } 
  else if (myLevel == 3) {
    // Four outside walls with gaps plus internal cross
    add_wall(gameMinX,gameMinY,gameCentreX-3,gameMinY);
    add_wall(gameCentreX+3,gameMinY,gameMaxX,gameMinY);
    add_wall(gameMaxX,gameMinY,gameMaxX,gameCentreY-3);
    add_wall(gameMaxX,gameCentreY+3,gameMaxX,gameMaxY);
    add_wall(gameMinX,gameMinY,gameMinX,gameCentreY-3);
    add_wall(gameMinX,gameCentreY+3,gameMinX,gameMaxY);
    add_wall(gameMinX,gameMaxY,gameCentreX-3,gameMaxY);
    add_wall(gameCentreX+3,gameMaxY,gameMaxX,gameMaxY);
    add_wall((gameMaxX/3), gameCentreY, (gameMaxX/3*2), gameCentreY);
    add_wall(gameCentreX, gameMaxY/3, gameCentreX, gameMaxY/3*2);
    snake_setup_snake(gameCentreX, gameCentreY-5);
  } 
  else if (myLevel == 4) {
    // Two walls splitting board into quarters
    add_wall(gameCentreX, gameMinY, gameCentreX, gameMaxX);
    add_wall(gameMinX, gameCentreY, gameMaxX, gameCentreY);
    snake_setup_snake(gameMinX+2, gameMinY+2);
  }
}

void snake_setup_game()
{
  points=0;
  level=1;
  clear_display();
  write_predef(LEVEL,1,1,7);
  write_number(level,8,12,7);
  int oldlevel = level;
  refresh_display(100);

  while ((PINC & 0x01) && (PINB & 0x1)) {  // Wait for button 1-5
    if (!(PINC & 0x04)) level--;
    if (!(PINC & 0x08)) level++;
    if (level==0) level=1;
    if (level != oldlevel) {
      clear_display();
      write_predef(LEVEL,1,1,7);
      write_number(level,8,12,7);
      refresh_display(50);
      oldlevel=level;
    }
    refresh_display(1);
  }
  snake_setup_level();
}

void snake_setup_level()
{
  clear_display();

  // 5 initial apples
  apples.Clear();
  for (byte i=0; i<5; i++) {
    snake_add_apple();
  }

  speed = 32-((points % levelMax)/2)-(2*level);
  snake_setup_board();  
}

